# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ga4gh/genotype_phenotype.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ga4gh import metadata_pb2 as ga4gh_dot_metadata__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ga4gh/genotype_phenotype.proto',
  package='ga4gh',
  syntax='proto3',
  serialized_pb=_b('\n\x1ega4gh/genotype_phenotype.proto\x12\x05ga4gh\x1a\x14ga4gh/metadata.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xc8\x01\n\x17PhenotypeAssociationSet\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ndataset_id\x18\x03 \x01(\t\x12\x36\n\x04info\x18\x04 \x03(\x0b\x32(.ga4gh.PhenotypeAssociationSet.InfoEntry\x1aG\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.ListValue:\x02\x38\x01\"f\n\x14\x45nvironmentalContext\x12\n\n\x02id\x18\x01 \x01(\t\x12-\n\x10\x65nvironment_type\x18\x02 \x01(\x0b\x32\x13.ga4gh.OntologyTerm\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\"\xa5\x02\n\x11PhenotypeInstance\x12\n\n\x02id\x18\x01 \x01(\t\x12!\n\x04type\x18\x02 \x01(\x0b\x32\x13.ga4gh.OntologyTerm\x12&\n\tqualifier\x18\x03 \x03(\x0b\x32\x13.ga4gh.OntologyTerm\x12)\n\x0c\x61ge_of_onset\x18\x04 \x01(\x0b\x32\x13.ga4gh.OntologyTerm\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x30\n\x04info\x18\x06 \x03(\x0b\x32\".ga4gh.PhenotypeInstance.InfoEntry\x1aG\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.ListValue:\x02\x38\x01\"\xbd\x01\n\x08\x45vidence\x12*\n\revidence_type\x18\x01 \x01(\x0b\x32\x13.ga4gh.OntologyTerm\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\'\n\x04info\x18\x03 \x03(\x0b\x32\x19.ga4gh.Evidence.InfoEntry\x1aG\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.ListValue:\x02\x38\x01\"\x8b\x03\n\x1b\x46\x65\x61turePhenotypeAssociation\x12\n\n\x02id\x18\x01 \x01(\t\x12$\n\x1cphenotype_association_set_id\x18\x02 \x01(\t\x12\x13\n\x0b\x66\x65\x61ture_ids\x18\x03 \x03(\t\x12!\n\x08\x65vidence\x18\x04 \x03(\x0b\x32\x0f.ga4gh.Evidence\x12+\n\tphenotype\x18\x05 \x01(\x0b\x32\x18.ga4gh.PhenotypeInstance\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12;\n\x16\x65nvironmental_contexts\x18\x07 \x03(\x0b\x32\x1b.ga4gh.EnvironmentalContext\x12:\n\x04info\x18\x08 \x03(\x0b\x32,.ga4gh.FeaturePhenotypeAssociation.InfoEntry\x1aG\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.ListValue:\x02\x38\x01\x62\x06proto3')
  ,
  dependencies=[ga4gh_dot_metadata__pb2.DESCRIPTOR,google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_PHENOTYPEASSOCIATIONSET_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='ga4gh.PhenotypeAssociationSet.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ga4gh.PhenotypeAssociationSet.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='ga4gh.PhenotypeAssociationSet.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=223,
  serialized_end=294,
)

_PHENOTYPEASSOCIATIONSET = _descriptor.Descriptor(
  name='PhenotypeAssociationSet',
  full_name='ga4gh.PhenotypeAssociationSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ga4gh.PhenotypeAssociationSet.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='ga4gh.PhenotypeAssociationSet.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dataset_id', full_name='ga4gh.PhenotypeAssociationSet.dataset_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info', full_name='ga4gh.PhenotypeAssociationSet.info', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_PHENOTYPEASSOCIATIONSET_INFOENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=94,
  serialized_end=294,
)


_ENVIRONMENTALCONTEXT = _descriptor.Descriptor(
  name='EnvironmentalContext',
  full_name='ga4gh.EnvironmentalContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ga4gh.EnvironmentalContext.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='environment_type', full_name='ga4gh.EnvironmentalContext.environment_type', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='ga4gh.EnvironmentalContext.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=296,
  serialized_end=398,
)


_PHENOTYPEINSTANCE_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='ga4gh.PhenotypeInstance.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ga4gh.PhenotypeInstance.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='ga4gh.PhenotypeInstance.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=223,
  serialized_end=294,
)

_PHENOTYPEINSTANCE = _descriptor.Descriptor(
  name='PhenotypeInstance',
  full_name='ga4gh.PhenotypeInstance',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ga4gh.PhenotypeInstance.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='ga4gh.PhenotypeInstance.type', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='qualifier', full_name='ga4gh.PhenotypeInstance.qualifier', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='age_of_onset', full_name='ga4gh.PhenotypeInstance.age_of_onset', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='ga4gh.PhenotypeInstance.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info', full_name='ga4gh.PhenotypeInstance.info', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_PHENOTYPEINSTANCE_INFOENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=401,
  serialized_end=694,
)


_EVIDENCE_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='ga4gh.Evidence.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ga4gh.Evidence.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='ga4gh.Evidence.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=223,
  serialized_end=294,
)

_EVIDENCE = _descriptor.Descriptor(
  name='Evidence',
  full_name='ga4gh.Evidence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='evidence_type', full_name='ga4gh.Evidence.evidence_type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='ga4gh.Evidence.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info', full_name='ga4gh.Evidence.info', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_EVIDENCE_INFOENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=697,
  serialized_end=886,
)


_FEATUREPHENOTYPEASSOCIATION_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='ga4gh.FeaturePhenotypeAssociation.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ga4gh.FeaturePhenotypeAssociation.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='ga4gh.FeaturePhenotypeAssociation.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=223,
  serialized_end=294,
)

_FEATUREPHENOTYPEASSOCIATION = _descriptor.Descriptor(
  name='FeaturePhenotypeAssociation',
  full_name='ga4gh.FeaturePhenotypeAssociation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ga4gh.FeaturePhenotypeAssociation.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='phenotype_association_set_id', full_name='ga4gh.FeaturePhenotypeAssociation.phenotype_association_set_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='feature_ids', full_name='ga4gh.FeaturePhenotypeAssociation.feature_ids', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='evidence', full_name='ga4gh.FeaturePhenotypeAssociation.evidence', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='phenotype', full_name='ga4gh.FeaturePhenotypeAssociation.phenotype', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='ga4gh.FeaturePhenotypeAssociation.description', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='environmental_contexts', full_name='ga4gh.FeaturePhenotypeAssociation.environmental_contexts', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info', full_name='ga4gh.FeaturePhenotypeAssociation.info', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_FEATUREPHENOTYPEASSOCIATION_INFOENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=889,
  serialized_end=1284,
)

_PHENOTYPEASSOCIATIONSET_INFOENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_struct__pb2._LISTVALUE
_PHENOTYPEASSOCIATIONSET_INFOENTRY.containing_type = _PHENOTYPEASSOCIATIONSET
_PHENOTYPEASSOCIATIONSET.fields_by_name['info'].message_type = _PHENOTYPEASSOCIATIONSET_INFOENTRY
_ENVIRONMENTALCONTEXT.fields_by_name['environment_type'].message_type = ga4gh_dot_metadata__pb2._ONTOLOGYTERM
_PHENOTYPEINSTANCE_INFOENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_struct__pb2._LISTVALUE
_PHENOTYPEINSTANCE_INFOENTRY.containing_type = _PHENOTYPEINSTANCE
_PHENOTYPEINSTANCE.fields_by_name['type'].message_type = ga4gh_dot_metadata__pb2._ONTOLOGYTERM
_PHENOTYPEINSTANCE.fields_by_name['qualifier'].message_type = ga4gh_dot_metadata__pb2._ONTOLOGYTERM
_PHENOTYPEINSTANCE.fields_by_name['age_of_onset'].message_type = ga4gh_dot_metadata__pb2._ONTOLOGYTERM
_PHENOTYPEINSTANCE.fields_by_name['info'].message_type = _PHENOTYPEINSTANCE_INFOENTRY
_EVIDENCE_INFOENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_struct__pb2._LISTVALUE
_EVIDENCE_INFOENTRY.containing_type = _EVIDENCE
_EVIDENCE.fields_by_name['evidence_type'].message_type = ga4gh_dot_metadata__pb2._ONTOLOGYTERM
_EVIDENCE.fields_by_name['info'].message_type = _EVIDENCE_INFOENTRY
_FEATUREPHENOTYPEASSOCIATION_INFOENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_struct__pb2._LISTVALUE
_FEATUREPHENOTYPEASSOCIATION_INFOENTRY.containing_type = _FEATUREPHENOTYPEASSOCIATION
_FEATUREPHENOTYPEASSOCIATION.fields_by_name['evidence'].message_type = _EVIDENCE
_FEATUREPHENOTYPEASSOCIATION.fields_by_name['phenotype'].message_type = _PHENOTYPEINSTANCE
_FEATUREPHENOTYPEASSOCIATION.fields_by_name['environmental_contexts'].message_type = _ENVIRONMENTALCONTEXT
_FEATUREPHENOTYPEASSOCIATION.fields_by_name['info'].message_type = _FEATUREPHENOTYPEASSOCIATION_INFOENTRY
DESCRIPTOR.message_types_by_name['PhenotypeAssociationSet'] = _PHENOTYPEASSOCIATIONSET
DESCRIPTOR.message_types_by_name['EnvironmentalContext'] = _ENVIRONMENTALCONTEXT
DESCRIPTOR.message_types_by_name['PhenotypeInstance'] = _PHENOTYPEINSTANCE
DESCRIPTOR.message_types_by_name['Evidence'] = _EVIDENCE
DESCRIPTOR.message_types_by_name['FeaturePhenotypeAssociation'] = _FEATUREPHENOTYPEASSOCIATION

PhenotypeAssociationSet = _reflection.GeneratedProtocolMessageType('PhenotypeAssociationSet', (_message.Message,), dict(

  InfoEntry = _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _PHENOTYPEASSOCIATIONSET_INFOENTRY,
    __module__ = 'ga4gh.genotype_phenotype_pb2'
    # @@protoc_insertion_point(class_scope:ga4gh.PhenotypeAssociationSet.InfoEntry)
    ))
  ,
  DESCRIPTOR = _PHENOTYPEASSOCIATIONSET,
  __module__ = 'ga4gh.genotype_phenotype_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.PhenotypeAssociationSet)
  ))
_sym_db.RegisterMessage(PhenotypeAssociationSet)
_sym_db.RegisterMessage(PhenotypeAssociationSet.InfoEntry)

EnvironmentalContext = _reflection.GeneratedProtocolMessageType('EnvironmentalContext', (_message.Message,), dict(
  DESCRIPTOR = _ENVIRONMENTALCONTEXT,
  __module__ = 'ga4gh.genotype_phenotype_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.EnvironmentalContext)
  ))
_sym_db.RegisterMessage(EnvironmentalContext)

PhenotypeInstance = _reflection.GeneratedProtocolMessageType('PhenotypeInstance', (_message.Message,), dict(

  InfoEntry = _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _PHENOTYPEINSTANCE_INFOENTRY,
    __module__ = 'ga4gh.genotype_phenotype_pb2'
    # @@protoc_insertion_point(class_scope:ga4gh.PhenotypeInstance.InfoEntry)
    ))
  ,
  DESCRIPTOR = _PHENOTYPEINSTANCE,
  __module__ = 'ga4gh.genotype_phenotype_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.PhenotypeInstance)
  ))
_sym_db.RegisterMessage(PhenotypeInstance)
_sym_db.RegisterMessage(PhenotypeInstance.InfoEntry)

Evidence = _reflection.GeneratedProtocolMessageType('Evidence', (_message.Message,), dict(

  InfoEntry = _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _EVIDENCE_INFOENTRY,
    __module__ = 'ga4gh.genotype_phenotype_pb2'
    # @@protoc_insertion_point(class_scope:ga4gh.Evidence.InfoEntry)
    ))
  ,
  DESCRIPTOR = _EVIDENCE,
  __module__ = 'ga4gh.genotype_phenotype_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.Evidence)
  ))
_sym_db.RegisterMessage(Evidence)
_sym_db.RegisterMessage(Evidence.InfoEntry)

FeaturePhenotypeAssociation = _reflection.GeneratedProtocolMessageType('FeaturePhenotypeAssociation', (_message.Message,), dict(

  InfoEntry = _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _FEATUREPHENOTYPEASSOCIATION_INFOENTRY,
    __module__ = 'ga4gh.genotype_phenotype_pb2'
    # @@protoc_insertion_point(class_scope:ga4gh.FeaturePhenotypeAssociation.InfoEntry)
    ))
  ,
  DESCRIPTOR = _FEATUREPHENOTYPEASSOCIATION,
  __module__ = 'ga4gh.genotype_phenotype_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.FeaturePhenotypeAssociation)
  ))
_sym_db.RegisterMessage(FeaturePhenotypeAssociation)
_sym_db.RegisterMessage(FeaturePhenotypeAssociation.InfoEntry)


_PHENOTYPEASSOCIATIONSET_INFOENTRY.has_options = True
_PHENOTYPEASSOCIATIONSET_INFOENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_PHENOTYPEINSTANCE_INFOENTRY.has_options = True
_PHENOTYPEINSTANCE_INFOENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_EVIDENCE_INFOENTRY.has_options = True
_EVIDENCE_INFOENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_FEATUREPHENOTYPEASSOCIATION_INFOENTRY.has_options = True
_FEATUREPHENOTYPEASSOCIATION_INFOENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
