# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ga4gh/allele_annotations.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ga4gh import metadata_pb2 as ga4gh_dot_metadata__pb2
from ga4gh import assay_metadata_pb2 as ga4gh_dot_assay__metadata__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ga4gh/allele_annotations.proto',
  package='ga4gh',
  syntax='proto3',
  serialized_pb=_b('\n\x1ega4gh/allele_annotations.proto\x12\x05ga4gh\x1a\x14ga4gh/metadata.proto\x1a\x1aga4gh/assay_metadata.proto\x1a\x1cgoogle/protobuf/struct.proto\"D\n\x0e\x41nalysisResult\x12\x13\n\x0b\x61nalysis_id\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\t\x12\r\n\x05score\x18\x03 \x01(\x05\"d\n\x0e\x41lleleLocation\x12\r\n\x05start\x18\x01 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x05\x12\x1a\n\x12reference_sequence\x18\x03 \x01(\t\x12\x1a\n\x12\x61lternate_sequence\x18\x04 \x01(\t\"k\n\x14VariantAnnotationSet\x12\n\n\x02id\x18\x01 \x01(\t\x12\x16\n\x0evariant_set_id\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12!\n\x08\x61nalysis\x18\x04 \x01(\x0b\x32\x0f.ga4gh.Analysis\"F\n\x0eHGVSAnnotation\x12\x0f\n\x07genomic\x18\x01 \x01(\t\x12\x12\n\ntranscript\x18\x02 \x01(\t\x12\x0f\n\x07protein\x18\x03 \x01(\t\"\xd8\x03\n\x10TranscriptEffect\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\nfeature_id\x18\x02 \x01(\t\x12\x17\n\x0f\x61lternate_bases\x18\x03 \x01(\t\x12$\n\x07\x65\x66\x66\x65\x63ts\x18\x04 \x03(\x0b\x32\x13.ga4gh.OntologyTerm\x12.\n\x0fhgvs_annotation\x18\x05 \x01(\x0b\x32\x15.ga4gh.HGVSAnnotation\x12,\n\rcdna_location\x18\x06 \x01(\x0b\x32\x15.ga4gh.AlleleLocation\x12+\n\x0c\x63\x64s_location\x18\x07 \x01(\x0b\x32\x15.ga4gh.AlleleLocation\x12/\n\x10protein_location\x18\x08 \x01(\x0b\x32\x15.ga4gh.AlleleLocation\x12/\n\x10\x61nalysis_results\x18\t \x03(\x0b\x32\x15.ga4gh.AnalysisResult\x12/\n\x04info\x18\n \x03(\x0b\x32!.ga4gh.TranscriptEffect.InfoEntry\x1aG\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.ListValue:\x02\x38\x01\"\x97\x02\n\x11VariantAnnotation\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\nvariant_id\x18\x02 \x01(\t\x12!\n\x19variant_annotation_set_id\x18\x03 \x01(\t\x12\x0f\n\x07\x63reated\x18\x04 \x01(\t\x12\x33\n\x12transcript_effects\x18\x05 \x03(\x0b\x32\x17.ga4gh.TranscriptEffect\x12\x30\n\x04info\x18\x06 \x03(\x0b\x32\".ga4gh.VariantAnnotation.InfoEntry\x1aG\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.ListValue:\x02\x38\x01\x62\x06proto3')
  ,
  dependencies=[ga4gh_dot_metadata__pb2.DESCRIPTOR,ga4gh_dot_assay__metadata__pb2.DESCRIPTOR,google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_ANALYSISRESULT = _descriptor.Descriptor(
  name='AnalysisResult',
  full_name='ga4gh.AnalysisResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='analysis_id', full_name='ga4gh.AnalysisResult.analysis_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='result', full_name='ga4gh.AnalysisResult.result', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score', full_name='ga4gh.AnalysisResult.score', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=189,
)


_ALLELELOCATION = _descriptor.Descriptor(
  name='AlleleLocation',
  full_name='ga4gh.AlleleLocation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start', full_name='ga4gh.AlleleLocation.start', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end', full_name='ga4gh.AlleleLocation.end', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='reference_sequence', full_name='ga4gh.AlleleLocation.reference_sequence', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='alternate_sequence', full_name='ga4gh.AlleleLocation.alternate_sequence', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=191,
  serialized_end=291,
)


_VARIANTANNOTATIONSET = _descriptor.Descriptor(
  name='VariantAnnotationSet',
  full_name='ga4gh.VariantAnnotationSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ga4gh.VariantAnnotationSet.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='variant_set_id', full_name='ga4gh.VariantAnnotationSet.variant_set_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='ga4gh.VariantAnnotationSet.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='analysis', full_name='ga4gh.VariantAnnotationSet.analysis', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=293,
  serialized_end=400,
)


_HGVSANNOTATION = _descriptor.Descriptor(
  name='HGVSAnnotation',
  full_name='ga4gh.HGVSAnnotation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='genomic', full_name='ga4gh.HGVSAnnotation.genomic', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='transcript', full_name='ga4gh.HGVSAnnotation.transcript', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='protein', full_name='ga4gh.HGVSAnnotation.protein', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=402,
  serialized_end=472,
)


_TRANSCRIPTEFFECT_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='ga4gh.TranscriptEffect.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ga4gh.TranscriptEffect.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='ga4gh.TranscriptEffect.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=876,
  serialized_end=947,
)

_TRANSCRIPTEFFECT = _descriptor.Descriptor(
  name='TranscriptEffect',
  full_name='ga4gh.TranscriptEffect',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ga4gh.TranscriptEffect.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='feature_id', full_name='ga4gh.TranscriptEffect.feature_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='alternate_bases', full_name='ga4gh.TranscriptEffect.alternate_bases', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='effects', full_name='ga4gh.TranscriptEffect.effects', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hgvs_annotation', full_name='ga4gh.TranscriptEffect.hgvs_annotation', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cdna_location', full_name='ga4gh.TranscriptEffect.cdna_location', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cds_location', full_name='ga4gh.TranscriptEffect.cds_location', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='protein_location', full_name='ga4gh.TranscriptEffect.protein_location', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='analysis_results', full_name='ga4gh.TranscriptEffect.analysis_results', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info', full_name='ga4gh.TranscriptEffect.info', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_TRANSCRIPTEFFECT_INFOENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=475,
  serialized_end=947,
)


_VARIANTANNOTATION_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='ga4gh.VariantAnnotation.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ga4gh.VariantAnnotation.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='ga4gh.VariantAnnotation.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=876,
  serialized_end=947,
)

_VARIANTANNOTATION = _descriptor.Descriptor(
  name='VariantAnnotation',
  full_name='ga4gh.VariantAnnotation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ga4gh.VariantAnnotation.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='variant_id', full_name='ga4gh.VariantAnnotation.variant_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='variant_annotation_set_id', full_name='ga4gh.VariantAnnotation.variant_annotation_set_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='created', full_name='ga4gh.VariantAnnotation.created', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='transcript_effects', full_name='ga4gh.VariantAnnotation.transcript_effects', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info', full_name='ga4gh.VariantAnnotation.info', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_VARIANTANNOTATION_INFOENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=950,
  serialized_end=1229,
)

_VARIANTANNOTATIONSET.fields_by_name['analysis'].message_type = ga4gh_dot_assay__metadata__pb2._ANALYSIS
_TRANSCRIPTEFFECT_INFOENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_struct__pb2._LISTVALUE
_TRANSCRIPTEFFECT_INFOENTRY.containing_type = _TRANSCRIPTEFFECT
_TRANSCRIPTEFFECT.fields_by_name['effects'].message_type = ga4gh_dot_metadata__pb2._ONTOLOGYTERM
_TRANSCRIPTEFFECT.fields_by_name['hgvs_annotation'].message_type = _HGVSANNOTATION
_TRANSCRIPTEFFECT.fields_by_name['cdna_location'].message_type = _ALLELELOCATION
_TRANSCRIPTEFFECT.fields_by_name['cds_location'].message_type = _ALLELELOCATION
_TRANSCRIPTEFFECT.fields_by_name['protein_location'].message_type = _ALLELELOCATION
_TRANSCRIPTEFFECT.fields_by_name['analysis_results'].message_type = _ANALYSISRESULT
_TRANSCRIPTEFFECT.fields_by_name['info'].message_type = _TRANSCRIPTEFFECT_INFOENTRY
_VARIANTANNOTATION_INFOENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_struct__pb2._LISTVALUE
_VARIANTANNOTATION_INFOENTRY.containing_type = _VARIANTANNOTATION
_VARIANTANNOTATION.fields_by_name['transcript_effects'].message_type = _TRANSCRIPTEFFECT
_VARIANTANNOTATION.fields_by_name['info'].message_type = _VARIANTANNOTATION_INFOENTRY
DESCRIPTOR.message_types_by_name['AnalysisResult'] = _ANALYSISRESULT
DESCRIPTOR.message_types_by_name['AlleleLocation'] = _ALLELELOCATION
DESCRIPTOR.message_types_by_name['VariantAnnotationSet'] = _VARIANTANNOTATIONSET
DESCRIPTOR.message_types_by_name['HGVSAnnotation'] = _HGVSANNOTATION
DESCRIPTOR.message_types_by_name['TranscriptEffect'] = _TRANSCRIPTEFFECT
DESCRIPTOR.message_types_by_name['VariantAnnotation'] = _VARIANTANNOTATION

AnalysisResult = _reflection.GeneratedProtocolMessageType('AnalysisResult', (_message.Message,), dict(
  DESCRIPTOR = _ANALYSISRESULT,
  __module__ = 'ga4gh.allele_annotations_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.AnalysisResult)
  ))
_sym_db.RegisterMessage(AnalysisResult)

AlleleLocation = _reflection.GeneratedProtocolMessageType('AlleleLocation', (_message.Message,), dict(
  DESCRIPTOR = _ALLELELOCATION,
  __module__ = 'ga4gh.allele_annotations_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.AlleleLocation)
  ))
_sym_db.RegisterMessage(AlleleLocation)

VariantAnnotationSet = _reflection.GeneratedProtocolMessageType('VariantAnnotationSet', (_message.Message,), dict(
  DESCRIPTOR = _VARIANTANNOTATIONSET,
  __module__ = 'ga4gh.allele_annotations_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.VariantAnnotationSet)
  ))
_sym_db.RegisterMessage(VariantAnnotationSet)

HGVSAnnotation = _reflection.GeneratedProtocolMessageType('HGVSAnnotation', (_message.Message,), dict(
  DESCRIPTOR = _HGVSANNOTATION,
  __module__ = 'ga4gh.allele_annotations_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.HGVSAnnotation)
  ))
_sym_db.RegisterMessage(HGVSAnnotation)

TranscriptEffect = _reflection.GeneratedProtocolMessageType('TranscriptEffect', (_message.Message,), dict(

  InfoEntry = _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRANSCRIPTEFFECT_INFOENTRY,
    __module__ = 'ga4gh.allele_annotations_pb2'
    # @@protoc_insertion_point(class_scope:ga4gh.TranscriptEffect.InfoEntry)
    ))
  ,
  DESCRIPTOR = _TRANSCRIPTEFFECT,
  __module__ = 'ga4gh.allele_annotations_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.TranscriptEffect)
  ))
_sym_db.RegisterMessage(TranscriptEffect)
_sym_db.RegisterMessage(TranscriptEffect.InfoEntry)

VariantAnnotation = _reflection.GeneratedProtocolMessageType('VariantAnnotation', (_message.Message,), dict(

  InfoEntry = _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), dict(
    DESCRIPTOR = _VARIANTANNOTATION_INFOENTRY,
    __module__ = 'ga4gh.allele_annotations_pb2'
    # @@protoc_insertion_point(class_scope:ga4gh.VariantAnnotation.InfoEntry)
    ))
  ,
  DESCRIPTOR = _VARIANTANNOTATION,
  __module__ = 'ga4gh.allele_annotations_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.VariantAnnotation)
  ))
_sym_db.RegisterMessage(VariantAnnotation)
_sym_db.RegisterMessage(VariantAnnotation.InfoEntry)


_TRANSCRIPTEFFECT_INFOENTRY.has_options = True
_TRANSCRIPTEFFECT_INFOENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_VARIANTANNOTATION_INFOENTRY.has_options = True
_VARIANTANNOTATION_INFOENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
