# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ga4gh/metadata_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ga4gh import metadata_pb2 as ga4gh_dot_metadata__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ga4gh/metadata_service.proto',
  package='ga4gh',
  syntax='proto3',
  serialized_pb=_b('\n\x1cga4gh/metadata_service.proto\x12\x05ga4gh\x1a\x14ga4gh/metadata.proto\">\n\x15SearchDatasetsRequest\x12\x11\n\tpage_size\x18\x01 \x01(\x05\x12\x12\n\npage_token\x18\x02 \x01(\t\"S\n\x16SearchDatasetsResponse\x12 \n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x0e.ga4gh.Dataset\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"\'\n\x11GetDatasetRequest\x12\x12\n\ndataset_id\x18\x01 \x01(\t2\x98\x01\n\x0fMetadataService\x12M\n\x0eSearchDatasets\x12\x1c.ga4gh.SearchDatasetsRequest\x1a\x1d.ga4gh.SearchDatasetsResponse\x12\x36\n\nGetDataset\x12\x18.ga4gh.GetDatasetRequest\x1a\x0e.ga4gh.Datasetb\x06proto3')
  ,
  dependencies=[ga4gh_dot_metadata__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_SEARCHDATASETSREQUEST = _descriptor.Descriptor(
  name='SearchDatasetsRequest',
  full_name='ga4gh.SearchDatasetsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='page_size', full_name='ga4gh.SearchDatasetsRequest.page_size', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='ga4gh.SearchDatasetsRequest.page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=61,
  serialized_end=123,
)


_SEARCHDATASETSRESPONSE = _descriptor.Descriptor(
  name='SearchDatasetsResponse',
  full_name='ga4gh.SearchDatasetsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='datasets', full_name='ga4gh.SearchDatasetsResponse.datasets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='ga4gh.SearchDatasetsResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=125,
  serialized_end=208,
)


_GETDATASETREQUEST = _descriptor.Descriptor(
  name='GetDatasetRequest',
  full_name='ga4gh.GetDatasetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset_id', full_name='ga4gh.GetDatasetRequest.dataset_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=210,
  serialized_end=249,
)

_SEARCHDATASETSRESPONSE.fields_by_name['datasets'].message_type = ga4gh_dot_metadata__pb2._DATASET
DESCRIPTOR.message_types_by_name['SearchDatasetsRequest'] = _SEARCHDATASETSREQUEST
DESCRIPTOR.message_types_by_name['SearchDatasetsResponse'] = _SEARCHDATASETSRESPONSE
DESCRIPTOR.message_types_by_name['GetDatasetRequest'] = _GETDATASETREQUEST

SearchDatasetsRequest = _reflection.GeneratedProtocolMessageType('SearchDatasetsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHDATASETSREQUEST,
  __module__ = 'ga4gh.metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchDatasetsRequest)
  ))
_sym_db.RegisterMessage(SearchDatasetsRequest)

SearchDatasetsResponse = _reflection.GeneratedProtocolMessageType('SearchDatasetsResponse', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHDATASETSRESPONSE,
  __module__ = 'ga4gh.metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.SearchDatasetsResponse)
  ))
_sym_db.RegisterMessage(SearchDatasetsResponse)

GetDatasetRequest = _reflection.GeneratedProtocolMessageType('GetDatasetRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETDATASETREQUEST,
  __module__ = 'ga4gh.metadata_service_pb2'
  # @@protoc_insertion_point(class_scope:ga4gh.GetDatasetRequest)
  ))
_sym_db.RegisterMessage(GetDatasetRequest)


# @@protoc_insertion_point(module_scope)
